# SOURCE CODE partly from Christoph Haenle
#
# (c) 1999-2000 Aberdeen University Electronics Research Lab
#
# THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
#
#    Modified by: Martin Koyabe, koyabe@erg.abdn.ac.uk
#           File: mftp_sim_general.tcl
#    Last update: May 24, 2000
#
# This script creates (1) source and (N) receiver nodes with asymmetric links.
# Each downlink experiences both random packet errors and losses (generated by error model). 



# Get input values

if {$argc != 5} {
    puts stderr {usage: ns mftp_sim2_general.tcl <GroupSize: 8,16,32..> <Number of receivers> <subnode step> <receivers in subnodes> <bit-error rate>}
    exit 1
}

# finish simulation routine

proc finish { } {
    global ns sender mftp_trace number_of_receivers
    $ns flush-trace
    #close $nf
    puts stdout "view output in mftp_sim_general.tr"
    puts stdout "Simulation successfully completed ...."
    exit 0
}

# setup simulator

set ns [new Simulator -multicast on]

# Set virtual classifier on

Simulator set node_factory_ VirtualClassifierNode

set verbose 1
set group_size [lindex $argv 0]
set number_of_receivers [lindex $argv 1]
set subnode_step [lindex $argv 2]
set subnode_number [lindex $argv 3]
set error_rate [lindex $argv 4]
set file_size 2048000
$ns set-address-format expanded
set group_addr [Node allocaddr]	;# allocate multicast group address


# Trace link events

set mftp_trace [open mftp_sim2_general.tr w]

# create the nam output file

set nf [open mftp_sim2_general.nam w]
$ns namtrace-all $nf


# Create Error Model

for {set k 0} {$k < $number_of_receivers} {incr k} {
    set error_model($k) [new ErrorModel]
    $error_model($k) unit bit 
    $error_model($k) set rate_ $error_rate
    $error_model($k) ranvar [new RandomVariable/Uniform]
    $error_model($k) drop-target [new Agent/Null]
}


# create network topology like this:
#
#
#                       21   22
#                        \   /
#                         \ /
#                          2--- 23
#                         /.
#                        / .
#              0--------1  .
#                        \ .
#                         \.
#                          n--- n1
#                         / \
#                        /   \
#                       n3    n2
#


# setup nodes:

for {set i 0} {$i < [expr $number_of_receivers+2]} {incr i} {
    set node_($i) [$ns node]
    
    if {[expr $i%$subnode_step] == 0 && $i > 1} {
	for {set j 1} {$j <= $subnode_number} {incr j} {
            set node_($i:$j) [$ns node]
        }
    }
}

# Creating links for WAN

if {$verbose} { puts "Creating links for WAN ...."; flush stdout }
$ns duplex-link $node_(0) $node_(1)  2500Kb 0ms DropTail

$ns queue-limit $node_(0) $node_(1)  100000
$ns queue-limit $node_(1) $node_(0)  100000

# getting the queue details

#set queue01 [[$ns link $node_(0) $node_(1)] queue]
#set queue10 [[$ns link $node_(1) $node_(0)] queue]

#puts "queue size node(0)-node(1): [$queue01 set limit_]"
#puts "queue size node(1)-node(0): [$queue10 set limit_]"

# Creating links for MAN with random delays

if {$verbose} { puts -nonewline "Creating links for LAN with (1..$number_of_receivers) nodes "; flush stdout }
for {set m 2} {$m < [expr $number_of_receivers+2]} {incr m} {
    $ns simplex-link $node_(1) $node_($m)  2500Kb [expr [expr int(rand()*1000)%150]+250]ms DropTail
    $ns simplex-link $node_($m) $node_(1)    64Kb [expr [expr int(rand()*1000)%150]+250]ms DropTail
    puts -nonewline "."; flush stdout

    if {[expr $m % $subnode_step] == 0 && $m > 1} {
#        puts stdout "m is: $m and step is: $subnode_step"
	for {set j 1} {$j <= $subnode_number} {incr j} {
            $ns duplex-link $node_($m) $node_($m:$j)  10000Kb [expr [expr int(rand()*1000)%5]+2]ms DropTail
            puts -nonewline "+"; flush stdout
        }
    }
}
puts " "; flush stdout

# DM set PruneTimeout 0.5
# set mrthandle [$ns mrtproto DM {}]

set mrthandle [$ns mrtproto CtrMcast {}]

# Insert link errors

if {$verbose} { puts -nonewline "Inserting link errors (1..$number_of_receivers) nodes "; flush stdout }

for {set l 2} {$l < [expr $number_of_receivers+2]} {incr l} {
    $ns link-lossmodel $error_model([expr $l-2]) $node_(1) $node_($l)
    puts -nonewline "."; flush stdout
}
puts " "; flush stdout


# color codes
$ns color 0 black
$ns color 1 blue
$ns color 2 purple
$ns color 3 green

# setup MFTP sender
set sender [new Agent/MFTP/Snd];        # new sender instance 
$sender set dst_addr_ $group_addr;      # set destination address
$sender set fid_ 2;                     # flow-id (for use in nam)
$sender set fileSize_ $file_size;       # size of file in bytes
$sender set dtusPerGroup_ $group_size;  # set transmission group size (in pkts/group)
$sender set dtusPerBlock_ 1424;         # set block size (in pkts/block), must be multiple of 8
$sender set dtuSize_ 1472;              # set packet size for transmission
$sender set interval_ 0.005;            # set interval between 2 packet transmissions (in s)
$sender trace $mftp_trace;              # optional set trace-file
$ns attach-agent $node_(0) $sender;     # attach MFTP-sender to node

set sender_addr  [$sender set agent_addr_]
set sender_port  [$sender set agent_port_]

# setting up MFTP receivers

for {set j 2} {$j < [expr $number_of_receivers+2]} {incr j} {

    if {[expr $j % $subnode_step] == 0} {
	for {set k 1} {$k <= $subnode_number} {incr k} {
	    set recv($j:$k) [new Agent/MFTP/Rcv/Stat]
	    $recv($j:$k) set dst_addr_ $group_addr
	    $recv($j:$k) set reply_addr_ $sender_addr;  # address to which packets are sent (unicast)
	    $recv($j:$k) set reply_port_ $sender_port;  # port to which packets are sent (unicast) 
	    $recv($j:$k) set fid_ 2
	    $recv($j:$k) set fileSize_ $file_size
	    $recv($j:$k) set dtusPerGroup_ $group_size
	    $recv($j:$k) set dtusPerBlock_ 1424
	    $recv($j:$k) set dtuSize_ 1472;              # set packet size for transmission
	    $recv($j:$k) trace $mftp_trace
	    $ns attach-agent $node_($j:$k) $recv($j:$k)
	    $ns at 0.1 "$recv($j:$k) start"
        }
    } else {
         set recv($j) [new Agent/MFTP/Rcv/Stat]
         $recv($j) set dst_addr_ $group_addr
         $recv($j) set reply_addr_ $sender_addr;  # address to which packets are sent (unicast)
	 $recv($j) set reply_port_ $sender_port;  # port to which packets are sent (unicast) 
         $recv($j) set fid_ 2
         $recv($j) set fileSize_ $file_size
         $recv($j) set dtusPerGroup_ $group_size
         $recv($j) set dtusPerBlock_ 1424
         $recv($j) set dtuSize_ 1472;              # set packet size for transmission
         $recv($j) trace $mftp_trace
         $ns attach-agent $node_($j) $recv($j)
         $ns at 0.1 "$recv($j) start"
   }
}

puts "Transmitting $file_size byte file with group-size $group_size"; flush stdout
puts "simulation is running..."; flush stdout
$ns at 0.0 "$sender start";                       # set time of start of transmission

# $ns at 99999.99 "[$ns clearMemTrace]"
# $ns at 100000.0 "exit 0"

$ns at 1000000.0 "finish"
$ns run

