# SOURCE CODE partly from Christoph Haenle
#
# (c) 1999-2000 Aberdeen University Electronics Research Lab
#
# THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
#
#    Modified by: Martin Koyabe, koyabe@erg.abdn.ac.uk
#           File: mftp_sim_general.tcl
#    Last update: Jan 24, 2000
#
# This script creates (1) source and (N) receiver nodes with asymmetric links.
# Each downlink experiences both random packet errors and losses (generated by error model). 



# Get input values

if {$argc != 3} {
    puts stderr {usage: ns mftp_sim_general.tcl <GroupSize: 8,16,32..> <Number_of_receivers> <bit-error rate>}
    exit 1
}

# finish simulation routine

proc finish { } {
    global ns sender mftp_trace number_of_receivers
    $ns flush-trace
    #close $nf
    puts stdout "view output in mftp_sim_[expr $number_of_receivers].tr"
    puts stdout "Simulation successfully completed ...."
    exit 0
}


# setup simulator

set ns [new Simulator -multicast on]

set verbose 1
set group_size [lindex $argv 0]
set number_of_receivers [lindex $argv 1]
set error_rate [lindex $argv 2]
set file_size 2000000
$ns set-address-format expanded
set group_addr [Node allocaddr]	;# allocate multicast group address
set node_address { }

# Trace link events

set srm_trace [open srm_sim_[expr $number_of_receivers].tr w]

# create the nam output file
# set nf [open mftp_sim_[expr $number_of_receivers].nam w]
# $ns namtrace-all $nf


# Create Error Model

for {set k 0} {$k < $number_of_receivers} {incr k} {
    set error_model($k) [new ErrorModel]
    $error_model($k) unit bit 
    $error_model($k) set rate_ $error_rate
    $error_model($k) ranvar [new RandomVariable/Uniform]
    $error_model($k) drop-target [new Agent/Null]
}


# create network topology like this:
#                          3
#                         /
#                        /
#              0--------1
#                        \
#                         \
#                          2


# setup nodes

set i 0
while {$i < [expr $number_of_receivers+2]} {
      set node_($i) [$ns node]
      if {$i > 1} {
         lappend node_address $i
      }
      incr i
}

# Creating links for WAN

if {$verbose} { puts "Creating links for WAN ...."; flush stdout }
$ns duplex-link $node_(0) $node_(1)  2500Kb 0ms DropTail

# Creating links for MAN with random delays

if {$verbose} { puts -nonewline "Creating links for LAN with (1..$number_of_receivers) nodes "; flush stdout }
for {set m 2} {$m < [expr $number_of_receivers+2]} {incr m} {
    set j [expr [expr int(rand()*1000)%150]+250]
    $ns simplex-link $node_(1) $node_($m)  2500Kb [expr $j]ms DropTail
    $ns simplex-link $node_($m) $node_(1)    64Kb [expr $j]ms DropTail
    puts -nonewline "."; flush stdout
}
puts " "; flush stdout

DM set PruneTimeout 1000
set mproto DM
set mrthandle [$ns mrtproto $mproto {}]

# Insert link errors

if {$verbose} { puts -nonewline "Inserting link errors (1..$number_of_receivers) nodes "; flush stdout }
for {set l 2} {$l < [expr $number_of_receivers+2]} {incr l} {
    $ns link-lossmodel $error_model([expr $l-2]) $node_(1) $node_($l)
    puts -nonewline "."; flush stdout
}
puts " "; flush stdout


# color codes
$ns color 0 black
$ns color 1 blue
$ns color 2 purple
$ns color 3 green

# register SRM sender:
set srm_sender [new Agent/SRM/Adaptive]
$srm_sender set dst_addr_ $group_addr
$srm_sender set fid_ 200
$srm_sender set sessionDelay_ 1
$srm_sender trace $srm_trace;           # optional set trace-file
$ns attach-agent $node_(0) $srm_sender; # attach SRM-sender to node
  
# SRM-sender is an SRM agent that has a traffic generator attached:
# scans e.g. SRM Send  CBR 3.52(in s) 50 (in kB)
# 1000(in Byte) 14400(in kbps)
    
  
# generate traffic-generator
set cbr [new Agent/CBR]
$cbr set packetSize_ $packet_size

# note: 1k=1000 (i.e. 64kbps is 64000 bit/s !!)
$cbr set interval_ 0.004; # in s
$cbr set fid_ 100

$srm_sender traffic-source $cbr
$srm_sender set packetSize_ 1024; # so repairs are correct
$ns at 0.0 "$srm_sender start; $cbr start"

set sender_addr  [$srm_sender set agent_addr_]
set sender_port  [$srm_sender set agent_port_]

# register SRM receivers:

for {set j 2} {$j < [expr $number_of_receivers+2]} {incr j} {
    set srm_recv($j) [new Agent/SRM/Adaptive]       
    $srm_recv($j) set dst_addr_ $group_addr
    $srm_recv($j) set fid_ $mcast_id
    $srm_recv($j) set sessionDelay_ 1
    $srm_recv($j) trace $srm_trace 
    $ns attach-agent $node_($j) $srm_recv($j)
        
    # SRM-receiver has a start-time when to join the session:
    # start_time is in s
    $ns at 0.1 "$srm_recv($j) start"

}

puts "Transmitting $file_size byte file with group-size $group_size"; flush stdout
puts "simulation is running..."; flush stdout
$ns at 0.0 "$srm_sender start;  $cbr start"             # set time of start of transmission
$ns at 100000.0 "finish"
$ns run
